import router from "@/router";
import { useUserStoreHook } from "@/store/modules/user";
import { usePermissionStoreHook } from "@/store/modules/permission";

import NProgress from "nprogress";
import "nprogress/nprogress.css";
NProgress.configure({ showSpinner: false }); // 进度条

const permissionStore = usePermissionStoreHook();

// 白名单路由
const whiteList = ["/login"];

router.beforeEach(async (to, from, next) => {
  NProgress.start();
  const hasToken = localStorage.getItem("accessToken");
  // 已登录
  if (hasToken) {
    // 已登录访问/login，跳转首页
    if (to.path === "/login") {
      next({ path: "/" });
      NProgress.done();
    } else {
      const userStore = useUserStoreHook();
      console.log("userStore.routeStatus", userStore.routeStatus);
      // 未加载过动态路由
      if (!userStore.routeStatus) {
        /**
         * userStore
         * ...
         * const userInfo = useStorage("userInfo", "");
         * ...
         * userInfo.value = JSON.stringify(data);
         * ...
         *   function getInfo() {
         *     return new Promise<UserInfo>((resolve, reject) => {
         * 			const userInfoStr = localStorage.getItem("userInfo");
         * 			const userInfo = JSON.parse(userInfoStr);
         * 			resolve({roles: userInfo.roles});
         *     });
         *   }
         */
        const { roles } = await userStore.getInfo();
        const accessRoutes = await permissionStore.generateRoutes(roles);
        accessRoutes.forEach((route) => {
          router.addRoute(route);
        });
        userStore.setRouteStatus(true);
        next({ ...to, replace: true });
      } else {
        next();
      }
    }
  }
  // 未登录
  else {
    // 未登录可以访问白名单页面
    if (whiteList.indexOf(to.path) !== -1) {
      next();
    } else {
      next(`/login?redirect=${to.path}`);
      NProgress.done();
    }
  }
});

router.afterEach(() => {
  NProgress.done();
});
